package com.sillycycle.bagleyd.ant2d;

/*
 * @(#)Ant2DFrame.java
 *
 * Copyright 2010 - 2020  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.net.URL;
import java.util.ArrayList;
import java.util.List;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JPopupMenu;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.UIManager;
import javax.swing.UnsupportedLookAndFeelException;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import com.sillycycle.bagleyd.util.ArgumentParser;
import com.sillycycle.bagleyd.util.ColumnLayout;
import com.sillycycle.bagleyd.util.ComponentUtil;
import com.sillycycle.bagleyd.util.Icon;
import com.sillycycle.bagleyd.util.IODialog;
import com.sillycycle.bagleyd.util.OKDialog;
import com.sillycycle.bagleyd.util.Spinner;
import com.sillycycle.bagleyd.util.TrafficLight;

/**
 * The <code>Ant2DFrame</code> class is to be embedded in a Web page
 * or run as an application.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/life.html
 */

public class Ant2DFrame extends JFrame implements ActionListener {
	private static final long serialVersionUID = 42L;
	public static final String NL = System.getProperty("line.separator");
	static final String[] COMMAND_STRINGS = new String[] {
		"Get Buffer", "Write Buffer", "Show Buffer",
		"Print", "Forms",
		"Stop", "Step (Yield)", "Run", "Clear",
		"Speed >", "Slow <", "Setup ~",
		"Description ?", "Features !", "References ^", "About",
		"Get File", "Write File", "Exit"};
	static final char[] COMMAND_CHARS = new char[] {
		'g', 'w', '*', 'p', 'f',
		's', 'y', 'r', 'c', '>', '<', '~',
		'?', '!', '^', 'a', 'G', 'W', 'x'};
	static final String[] TRAFFIC_STRINGS = new String[] {
		"Stop", "Step", "Run", "Clear"
	};
	static final char[] TRAFFIC_CHARS = new char[] {
		's', 'y', 'r', 'c'
	};
	static final Color[] TRAFFIC_COLORS = new Color[] {
		Color.red, Color.yellow, Color.green, Color.blue // For Jimmi
	};

	static ArgumentParser argumentParser = null;
	Icon mediumIcon = null;
	Icon largeIcon = null;
	int widthValue = 806;
	int heightValue = 456;
	static int xValue = 0;
	static int yValue = 0;
	static int ruleValue = Ant2DRules.MIN_RULE;
	Color bgColor = Color.gray.darker();
	boolean controlValue = true;
	int speedValue = Ant2DInterface.DEFAULT_SPEED;
	boolean debug = false;
	Ant2D game = null;
	JPanel topPanel = new JPanel();
	TrafficLight trafficLight = new TrafficLight(TRAFFIC_STRINGS,
		TRAFFIC_CHARS, TRAFFIC_COLORS);
	transient SpinnerModel ruleModel = new SpinnerNumberModel(ruleValue,
		Ant2DRules.MIN_RULE, Ant2DRules.MAX_RULE, 1);
	transient JSpinner ruleSpinner = new JSpinner(ruleModel);
	JTextField genTextField = new JTextField(5);
	JTextField cellsTextField = new JTextField(5);
	OKDialog aDialog = null;
	StringBuilder stringBuilderSave;
	Ant2DDialog gameDialog = null;
	IODialog ioDialog = null;
	static int neighborIndex = Ant2DInterface.DEFAULT_NEIGHBOR_INDEX;
	JMenuBar menuBar;
	JMenuItem detachMenuItem;
	JPanel mainPanel = new JPanel();
	JPanel secondaryPanel = new JPanel();
	JFrame frame;
	Spinner speedChanger = null;
	//Slider speedChanger = null;

	public String writeAntString() {
		return this.stringBuilderSave.toString();
	}

	public void getAntString(String s) {
		game.stringSave = s;
	}

	public void startGame() {
		game.start();
	}

	public void endGame() {
		game.setPause(true);
	}

	public void pauseGame() {
		game.setPause(!game.getPause());
	}

	public void clearGame() {
		game.clearField(true);
		game.start();
	}

	public void processKey(char key) {
		game.processKey(key);
	}

	public synchronized void setDelay(int i) {
		this.speedValue = i;
		if (speedChanger != null)
			speedChanger.setValue(Ant2DInterface.MAX_SPEED +
				Ant2DInterface.MIN_SPEED - 1 - i);
	}

	public void setColumns(int i) {
		if (game == null) {
			gameDialog.setColumns(i);
		} else {
			game.setColumns(i);
			gameDialog.setColumns(game.getColumns());
		}
	}

	public static int getColumns() {
		return Ant2DDialog.getColumns();
	}

	public void setNeighborIndex(int i) {
		gameDialog.setNeighborIndex(i);
		if (game != null)
			game.setNeighborIndex(i);
	}

	public static int getNeighborIndex() {
		return Ant2DDialog.getNeighborIndex();
	}

	public void setRules(int oldNeighborIndex, int newNeighborIndex) {
		if (game != null && oldNeighborIndex != newNeighborIndex) {
			game.setNeighbors(newNeighborIndex);
			game.ruleChange();
			game.clearField(true);
			game.eraseFrame();
			game.initTable();
			game.prepopulate(true);
			game.resizeField();
		}
	}

	public void setGrid(boolean flag) {
		gameDialog.setGrid(flag);
		if (game != null)
			game.setGrid(flag);
	}

	public static boolean getGrid() {
		return Ant2DDialog.getGrid();
	}

	/*public void setInvert(boolean flag) {
		gameDialog.setInvert(flag);
		if (game != null)
			game.setInvert(flag);
	}

	public boolean getInvert() {
		return gameDialog.getInvert();
	}*/

	public void setRandomOrient(boolean flag) {
		gameDialog.setRandomOrient(flag);
		if (game != null)
			game.setRandomOrient(flag);
	}

	public static boolean getRandomOrient() {
		return Ant2DDialog.getRandomOrient();
	}

	public void setWrap(boolean flag) {
		gameDialog.setWrap(flag);
		if (game != null)
			game.setWrap(flag);
	}

	public static boolean getWrap() {
		return Ant2DDialog.getWrap();
	}

	public void setCellVisible(boolean flag) {
		gameDialog.setCellVisible(flag);
		if (game != null)
			game.setCellVisible(flag);
	}

	public static boolean getCellVisible() {
		return Ant2DDialog.getCellVisible();
	}

	public void setUseTable(boolean flag) {
		gameDialog.setUseTable(flag);
		if (game != null)
			game.setUseTable(flag);
	}

	public static boolean getUseTable() {
		return Ant2DDialog.getUseTable();
	}

	public void setEyes(boolean flag) {
		gameDialog.setEyes(flag);
		if (game != null)
			game.setEyes(flag);
	}

	public static boolean getEyes() {
		return Ant2DDialog.getEyes();
	}

	public void setSharpTurn(boolean flag) {
		gameDialog.setSharpTurn(flag);
		if (game != null)
			game.setSharpTurn(flag);
	}

	public static boolean getSharpTurn() {
		return Ant2DDialog.getSharpTurn();
	}

	public void setTruchet(boolean flag) {
		gameDialog.setTruchet(flag);
		if (game != null)
			game.setTruchet(flag);
	}

	public static boolean getTruchet() {
		return Ant2DDialog.getTruchet();
	}

	public String getOpt(String string) {
		return argumentParser.getOption(string);
	}

	public int getOpt(String param, int value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Integer.parseInt(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public boolean getOpt(String param, boolean value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			if (string.equalsIgnoreCase("true") ||
					string.equalsIgnoreCase("yes") ||
					string.equalsIgnoreCase("on"))
				return true;
			else if (string.equalsIgnoreCase("false") ||
					string.equalsIgnoreCase("no") ||
					string.equalsIgnoreCase("off"))
				return false;
			return (Integer.parseInt(string) != 0);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public Color getOpt(String param, Color value) {
		String string = getOpt(param);

		if (string == null)
			return value;
		try {
			return Color.decode(string);
		} catch (NumberFormatException e) {
			return value; // ignore garbage
		}
	}

	public void getOpts() {
		int oldValue, newValue;
		int neighborsValue;
		int delayValue;
		int oldNeighborIndex;

		widthValue = getOpt("windowWidth", widthValue);
		heightValue = getOpt("windowHeight", heightValue);
		xValue = getOpt("windowX", xValue);
		yValue = getOpt("windowY", yValue);
		bgColor = getOpt("bgColor", bgColor);
		controlValue = getOpt("control", controlValue);
		delayValue = Ant2DInterface.MAX_SPEED +
			Ant2DInterface.MIN_SPEED - 1 - speedValue;
		delayValue = getOpt("delay", delayValue);
		speedValue = Ant2DInterface.MAX_SPEED +
			Ant2DInterface.MIN_SPEED - 1 - delayValue;
		oldValue = Ant2DDialog.getColumns();
		newValue = getOpt("columns", oldValue);
		if (newValue < Ant2DInterface.MIN_COLUMNS)
			newValue = Ant2DInterface.MIN_COLUMNS;
		if (oldValue != newValue)
			gameDialog.setColumns(newValue);
		neighborsValue = getOpt("neighbors",
			Ant2DRules.indexToNeighbor[Ant2DDialog.getNeighborIndex()]);
		oldNeighborIndex = neighborIndex;
		neighborIndex = Ant2DRules.neighborToIndex(neighborsValue);
		if (oldNeighborIndex != neighborIndex) {
			gameDialog.setNeighborIndex(neighborIndex);
		}
		gameDialog.setRandomOrient(getOpt("randomOrient",
			Ant2DDialog.getRandomOrient()));
		gameDialog.setGrid(getOpt("grid",
			Ant2DDialog.getGrid()));
		gameDialog.setWrap(getOpt("wrap",
			Ant2DDialog.getWrap()));
		gameDialog.setCellVisible(getOpt("visible",
			Ant2DDialog.getCellVisible()));
		gameDialog.setUseTable(getOpt("useTable",
			Ant2DDialog.getUseTable()));
		gameDialog.setEyes(getOpt("eyes",
			Ant2DDialog.getEyes()));
		gameDialog.setSharpTurn(getOpt("sharpTurn",
			Ant2DDialog.getSharpTurn()));
		gameDialog.setTruchet(getOpt("truchet",
			Ant2DDialog.getTruchet()));
	}

	public void getOpts2() {
		int oldValue = ruleValue;

		ruleValue = getOpt("rule", ruleValue);
		if (oldValue != ruleValue) {
			ruleSpinner.setValue(ruleValue);
			game.setRule(ruleValue);
		}
	}

	public void setupGame() {
		if (gameDialog != null) {
			gameDialog.setVisible(true);
		}
	}

	public void showBuffer() {
		if (ioDialog != null) {
			ioDialog.setVisible(true);
		}
	}

	public void showMessage(String msg) {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Ant2D", msg, "OK",
				mediumIcon, false);
			aDialog.setBounds(this.getSize().width / 2,
				this.getSize().height / 2, 350, 120);
			aDialog.setVisible(true);
		}
	}

	public void descriptionHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Description",
"Yet another implementation of Langton's Ant2Ds.  This one allows not " + NL +
"only square life but also hexagonal tiling." + NL +
"When running from web the \"write\" and \"read\" use cookies to output " + NL +
"data, else it uses \"" + Ant2DCanvas.DATAFILE + "\" file." + NL +
"Holding Control key down and use mouse to copy and paste." + NL +
"Can also manipulate life form by rotating and reflecting." + NL +
"See https://www.sillycycle.com/lifeNotes.html for TODO list. " + NL +
"Probably lots of other things need to be done too, let me know." + NL,
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void featuresHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"Features",
"Press mouse button on a on cell to erase (drag to do multiple)." + NL +
" " + NL +
"Press mouse button on a off cell to turn it on (drag to do multiple)." + NL +
" " + NL +
"Press \"g\" key to get (read) a saved life form to screen." + NL +
" " + NL +
"Press \"w\" key to save (write) a life form to screen." + NL +
" " + NL +
"Press \"G\" key to get (read) a saved life form to file." + NL +
" " + NL +
"Press \"W\" key to save (write) a life form to file." + NL +
" " + NL +
"Press \"P\" or \"p\" keys to print a life form." + NL +
" " + NL +
"Press \"C\" or \"c\" keys to clear." + NL +
" " + NL +
"Press \"O\" or \"o\" keys to wrap." + NL +
" " + NL +
"Press \"#\" key to toggle grid." + NL +
" " + NL +
"Press \">\" or \".\" keys to speed up ant." + NL +
" " + NL +
"Press \"<\" or \",\" keys to slow down ant." + NL +
" " + NL +
"Press \"]\" or \"}\" keys to increase the column/row " +
"size." + NL +
" " + NL +
"Press \"[\" or \"{\" keys to decrease the column/row " +
"size." + NL +
" " + NL +
"Press \"Y\" or \"y\" keys to step." + NL +
" " + NL +
"Press \"R\", \"r\", or \"|\" key to toggle run." + NL +
" " + NL +
"Press \"S\" or \"s\" key to toggle stop run." + NL +
" " + NL +
"Press \"~\" key for setup." + NL +
" " + NL +
"Press \"Esc\" key to hide and pause program." + NL +
" " + NL +
"Press \"Q\", \"q\", or \"CTRL-C\" keys to kill program if an application." + NL,
/* "The ant2d may be resized.  Cells will reshape depending on" + NL +
"the room they have." + NL + */
				"OK", largeIcon, true);
			aDialog.setBackground(Color.white);
			aDialog.setVisible(true);
		}
	}

	public void referencesHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"References",
"xlock ant2d source code" + NL +
"https://www.sillycycle.com/xlockmore.html" + NL +
" " + NL +
"New kinds of ants from http://havoc.gtf.gatech.edu/austin" + NL +
" " + NL +
"Dewdney, A.K., \"The Armchair Universe, Computer Recreations from the" + NL +
"Pages of Scientific American Magazine\", W.H. Freedman and Company," + NL +
"New York, 1988 (February 1987 p 121)" + NL +
" " + NL +
"Stewart, Ian, \"Mathematical Recreations, Scientific American" + NL +
"Jul 1994 pp 104-107",
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public void aboutHelp() {
		if (aDialog == null || !OKDialog.getActive()) {
			aDialog = new OKDialog(ComponentUtil.findJFrame(this),
				"About",
"Java Ant2DFrame Version 8.3.0" + NL +
"Send bugs (reports or fixes) to the author:" +
"David Bagley <bagleyd AT verizon.net>" + NL +
"The latest version is at: " +
"https://www.sillycycle.com/life.html" + NL,
				"OK", mediumIcon, false);
			aDialog.setVisible(true);
		}
	}

	public Icon getIcon(String string, int sx, int sy) {
		URL url = Ant2DFrame.class.getResource("/" + string);

		if (url == null)
			return null;
		return new Icon(Toolkit.getDefaultToolkit().
			getImage(url), sx, sy);
	}

	@Override
	public void setSize(int width, int height) {
		if (game == null)
			return;
		if (width != widthValue || height != heightValue) {
			game.setBounds(0, 0, width, height);
			game.resizeField();
			widthValue = width;
			heightValue = height;
			game.drawField(true);
		}
		/*repaint();*/ // causes flashing
	}

	@Override
	public void paint(Graphics g) {
		setSize(secondaryPanel.getSize().width,
			secondaryPanel.getSize().height - topPanel.getSize().height);
		super.paint(g);
	}

	public String readTextArea() {
		return ioDialog.getText();
	}

	public void writeTextArea(String s) {
		ioDialog.setText(s);
	}

	public void shuffleDown() {
		ComponentUtil.findJFrame(this).toBack();
	}

	public void quit() {
		System.exit(0);
	}

	public void displayMessage(String string) {
		if (frame != null) {
			if (string == null || "".equals(string))
				frame.setTitle(Ant2DInterface.TITLE);
			else
				frame.setTitle(Ant2DInterface.TITLE +
					": " + string);
		}
	}

	@Override
	public void requestFocus() {
		if (game != null)
			game.requestFocus();
	}

	public void updateGen(int gen) {
		if (genTextField == null)
			return;
		genTextField.setText(String.valueOf(gen));
	}

	public void updateCells(int cells) {
		if (cellsTextField == null)
			return;
		cellsTextField.setText(String.valueOf(cells));
	}

	public void initPopup() {
		if (game != null)
			game.initPopup();
	}

	public Image getImage(String fileName) {
		return Toolkit.getDefaultToolkit().getImage(fileName);
	}

	class PressActionListener implements ActionListener {
		@Override
		public void actionPerformed(ActionEvent event) {
			processKey(event.getActionCommand().charAt(0));
		}
	}

	class RuleChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = ruleValue;

			ruleValue = (Integer) ruleSpinner.getValue();
			// if is OK here since the next one should be OK
			if (Ant2DRules.boring(ruleValue) &&
					!getUseTable() &&
					oldValue != ruleValue) {
				if (oldValue < ruleValue)
					ruleValue++;
				else // if (oldValue > ruleValue)
					ruleValue--;
				ruleSpinner.setValue(ruleValue);
			}
			if (oldValue != ruleValue) {
				game.setRule(ruleValue);
			}
		}
	}

	class SpeedChangeListener implements ChangeListener {
		@Override
		public void stateChanged(ChangeEvent event) {
			int oldValue = speedValue;

			speedValue = speedChanger.getValue();
			if (oldValue != speedValue) {
				game.setDelay(Ant2DInterface.MAX_SPEED +
					Ant2DInterface.MIN_SPEED - 1 -
					speedValue);
			}
		}
	}

	@Override
	public void actionPerformed(ActionEvent event) {
		String command = event.getActionCommand();

		for (int opt = 0; opt < COMMAND_STRINGS.length; opt++) {
			if (command.equals(COMMAND_STRINGS[opt])) {
				processKey(COMMAND_CHARS[opt]);
				return;
			}
		}
	}

	public Ant2DFrame(String title) {
		JPanel controlPanel = new JPanel();
		JPanel textPanel = new JPanel();
		JPanel genPanel = new JPanel();
		JPanel canvasPanel = new JPanel();

		try {
			mediumIcon = getIcon(Ant2DInterface.ICONS[Ant2DInterface.Icons.X32.ordinal()], 32, 32);
			largeIcon = getIcon(Ant2DInterface.ICONS[Ant2DInterface.Icons.X48.ordinal()], 48, 48);
		} catch (Exception e) {
			largeIcon = null;
		}
		gameDialog = new Ant2DDialog(this,
			ComponentUtil.findJFrame(this),
			Ant2DInterface.TITLE + ": Setup", "DONE");
		ioDialog = new IODialog(ComponentUtil.findJFrame(this),
			Ant2DInterface.TITLE + ": Setup", "Life Form", "DONE");
		getOpts();
		controlPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 3));
		topPanel.setLayout(new ColumnLayout(0, 0, 0,
			ColumnLayout.LEFT));
		textPanel.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 3));
		mainPanel.setLayout(new BorderLayout());
		mainPanel.setBackground(Color.white);
		if (controlValue) {
			menuBar = new JMenuBar();
			JMenu menu;
			JMenuItem menuItem;

			JPopupMenu.setDefaultLightWeightPopupEnabled(false);
			menu = new JMenu("File");
			menuItem = new JMenuItem("Get Buffer");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_G);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Write Buffer");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_W);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Show Buffer");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Print");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_P);
			menuItem.addActionListener(this);
			menu.addSeparator();
			menuItem = new JMenuItem("Get File");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Write File");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menu.addSeparator();
			menuItem = new JMenuItem("Exit");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_X);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Play");
			menuItem = new JMenuItem("Stop");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_S);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Step (Yield)");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_Y);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Run");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_R);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Clear");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_C);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Speed >");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Slow <");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Setup ~");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			menu = new JMenu("Help");
			menuItem = new JMenuItem("Description ?");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("Features !");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("References ^");
			menu.add(menuItem);
			menuItem.addActionListener(this);
			menuItem = new JMenuItem("About");
			menu.add(menuItem);
			menuItem.setMnemonic(KeyEvent.VK_A);
			menuItem.addActionListener(this);
			menuBar.add(menu);
			//setJMenuBar(menuBar); // needs to be in a panel
			mainPanel.add(menuBar, BorderLayout.PAGE_START);
			controlPanel.add(trafficLight);
			genPanel.add(new JLabel("Rule:"));
			genPanel.add(ruleSpinner);
			genPanel.add(new JLabel("Generation:"));
			genPanel.add(genTextField);
			genTextField.setEditable(false);
			genPanel.add(new JLabel("Cells:"));
			genPanel.add(cellsTextField);
			cellsTextField.setEditable(false);
			speedChanger = new Spinner("Speed",
				speedValue, Ant2DInterface.MIN_SPEED,
				Ant2DInterface.MAX_SPEED);
			/*speedChanger = new Slider("Speed",
				speedValue, Ant2DInterface.MIN_SPEED,
				Ant2DInterface.MAX_SPEED);
			speedChanger.setWidth(120);*/
			textPanel.add(genPanel);
			controlPanel.add(speedChanger);
			gameDialog.setNeighborIndex(neighborIndex);
			genTextField.setText("0");
			cellsTextField.setText("0");
			topPanel.add(controlPanel);
			topPanel.add(textPanel);
			secondaryPanel.setLayout(new BorderLayout());
			secondaryPanel.add(topPanel, BorderLayout.PAGE_START);
		}
		game = new Ant2D(this, speedValue,
			Ant2DDialog.getColumns(),
			Ant2DRules.indexToNeighbor[Ant2DDialog.getNeighborIndex()],
			Ant2DDialog.getRandomOrient(),
			Ant2DDialog.getGrid(), Ant2DDialog.getWrap(),
			Ant2DDialog.getCellVisible(), Ant2DDialog.getUseTable(),
			Ant2DDialog.getEyes(), Ant2DDialog.getSharpTurn(),
			Ant2DDialog.getTruchet(), bgColor);
		canvasPanel.add(game);
		secondaryPanel.add(canvasPanel, BorderLayout.CENTER);
		mainPanel.add(secondaryPanel, BorderLayout.CENTER);
		getContentPane().add(mainPanel, BorderLayout.CENTER);
		if (controlValue) {
			trafficLight.addActionListener(new PressActionListener());
			ruleSpinner.addChangeListener(new RuleChangeListener());
			speedChanger.addChangeListener(new SpeedChangeListener());
		}
		getOpts2();
		game.setBounds(0, 0, widthValue, heightValue);
		//show();
		setTitle(title);
		game.requestFocus();
		game.setPause(true);
		/* game.setSize(getSize().width,
			getSize().height - topPanel.getSize().height); */
		//game.setSize(widthValue, heightValue);
	}

	public static void main(String[] args) {
		String laf = UIManager.getCrossPlatformLookAndFeelClassName();

		try {
			UIManager.setLookAndFeel(laf);
		} catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException e) {
			e.printStackTrace();
			System.exit(1);
		}
		List<Image> iconList = new ArrayList<>();
		for (String icon : Ant2DInterface.ICONS) {
			URL url = Ant2DFrame.class.getResource("/" + icon);
			if (url != null)
				iconList.add(Toolkit.getDefaultToolkit().getImage(url));
		}

		argumentParser = new ArgumentParser(args);
		if (argumentParser.hasOption("?")) {
			System.out.println("usage:" + NL +
"\t[-windowWidth={int}] [-windowHeight={int}]" + NL +
"\t[-windowX={int}] [-windowY={int}]" + NL +
"\t[-bgColor={int}] [-control={bool}]" + NL +
"\t[-columns={int}] [-neighbors={int}] [-rule={string}]" + NL +
"\t[-randomOrient={bool}] [-grid={bool}] [-wrap={bool}]" + NL +
"\t[-visible={bool}] [-useTable={bool}] [-eyes={bool}]" + NL +
"\t[-sharpTurn={bool}] [-truchet={bool}] [-delay={int}]");
			return;
		}
		Ant2DFrame frame = new Ant2DFrame(Ant2DInterface.TITLE);
		frame.addWindowListener(new WindowAdapter() {
			@Override
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		});
		if (!iconList.isEmpty()) {
			frame.setIconImages(iconList);
		}
		frame.pack();
		frame.game.requestFocusInWindow();
		frame.setLocation(Ant2DFrame.xValue, Ant2DFrame.yValue);
		frame.setVisible(true);
	}
}
