package com.sillycycle.bagleyd.ant2d;

/*
 * @(#)Ant2DRules.java
 *
 * Copyright 2010 - 2016  David A. Bagley, bagleyd AT verizon.net
 *
 * All rights reserved.
 *
 * Permission to use, copy, modify, and distribute this software and
 * its documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and
 * that both that copyright notice and this permission notice appear in
 * supporting documentation, and that the name of the author not be
 * used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission.
 *
 * This program is distributed in the hope that it will be "useful",
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */

import java.util.ArrayList;
import java.util.HashMap;

import com.sillycycle.bagleyd.util.FiniteStateMachine;

/**
 * The <code>Ant2DRules</code> class handles the rules of Ant2D's Ant2Ds.
 *
 * @author	David A. Bagley
 * @author	bagleyd AT verizon.net
 * @author	https://www.sillycycle.com/life.html
 */

public class Ant2DRules {
	static final int MAX_NEIGHBORS = 12;
	static final int MAX_NEIGHBOR_INDEXES = 10;
	//static final int MAX_NEIGHBORS = 8;
	//static final int MAX_NEIGHBOR_INDEXES = 5;
	static final int BASE = 10;
	static final boolean DEBUG = false;
	static final int MIN_RULE = 2;
	static final int MAX_RULE = 254;
	int ncolors = 8;
	int nstates = 1;

	private int neighborIndex;
	//private String ruleName;
	static int turingRule = MIN_RULE;

	public Ant2DRules(/*String ruleName*/) {
		//this.ruleName = ruleName;
	}

	/*public Ant2DRules(int neighbors, String ruleName) {
		this.neighborIndex = neighborToIndex(neighbors);
		//this.ruleName = ruleName;
		init();
	}*/

	public Ant2DRules(int neighbors) {
		this.neighborIndex = neighborToIndex(neighbors);
		init();
	}

	protected static String toString(int[] values) {
		StringBuilder string = new StringBuilder();

		for (int i = 0; i < values.length; i++) {
			if (i != 0)
				string.append(", ");
			string.append(i).append(": ").append(values[i]);
		}
		return string.toString();
	}

	public int getNeighbors() {
		return indexToNeighbor[neighborIndex];
	}

	public void setNeighborIndex(int neighborIndex) {
		this.neighborIndex = neighborIndex;
	}

	public void setNeighbors(int neighbors) {
		this.neighborIndex = neighborToIndex(neighbors);
	}

	public static String getRuleName(int neighborIndexValue) {
		return neighborIndexStrings[neighborIndexValue];
		//return Integer.toHexString(turingRule);
	}

	// Array of Generic Hash Maps
	ArrayList<HashMap<String, Integer>> arrayHashMap = new ArrayList<>();
//http://forums.sun.com/thread.jspa?threadID=530823
	final void init() {
		if (!arrayHashMap.isEmpty())
			arrayHashMap.clear();
		for (int i = 0; i < MAX_NEIGHBOR_INDEXES; i++) {
//System.out.println("Ant2DRules.init: " + " " + i);
			arrayHashMap.add(new HashMap<String, Integer>());
			//arrayHashMap.add(new HashMap<>());
//System.out.println("Ant2DRules.init: " + " " + neighborIndex + " " + ant2dRules[neighborIndex].length);
		//for (int j = 0; j < ant2dRules[i].length; j++) {
//System.out.println("Ant2DRules.init: " + " " + indexToNeighbor[neighborIndex] + ant2dRules[neighborIndex][i].ruleName + " " + i);
		//	arrayHashMap.get(i).put(ant2dRules[i][j].ruleName, Integer.valueOf(j));
		//}
		}
	}

	static final int[] neighborIndexToPolygon = {
		3, 4, 5, 6, 4,
		6, 3, 5, /*4,*/ 3, 3, 3
	};

	static int neighborToPolygon(int neighbors) {
		switch (neighbors) {
		case 3: /* triangle */
		case -6:
		case 9:
		case -9:
		case 12:
			return 3;
		case 4: /* square */
		case 8:
			return 4;
		case 5: /* pentagon, Cairo Tiling */
		case 7:
			return 5;
		case 6: /* hexagon */
			return 6;
		default:
			System.out.println("no polygon known for " +
				neighbors + " neighbors");
			return 4;
		}
	}

	static final String[] neighborIndexStrings = {
		"3, Triangular, von Neuman",
		"4, Square, von Neuman",
		"5, Pentagon, Cairo Tiling",
		"6, Hexagonal",

		"8, Square, Moore",

		// these are not ready for prime time
		//"6, Triangular, sides and far corners",
		//"7, Pentagonal, Moore, Cairo Tiling",
		//"9, Triangular, exclude far corners",
		//"9, Triangular, exclude sides",
		//"12, Triangular, Moore"
	};

	static final int[] indexToNeighbor = {
		3, 4, 5, 6,  8,  -6, 7, /*8,*/ 9, -9, 12
	};

	static int neighborToIndex(int neighbors) {
		switch (neighbors) {
		case 3: /* von Neuman Neighbor Count */
			return 0;
		case 4: /* von Neuman Neighbor Count */
			return 1;
		case 5: /* Cairo Tiling */
			return 2;
		case 6:
			return 3;

		case 8: /* Moore Neighbor Count */
			return 4;

		case -6: /* Moore Neighbor Count just sides and far corners */
			return 5;
		case 7: /* Cairo Tiling, Moore Neighbor Count */
			return 6;

		case 9: /* Moore Neighbor Count but excluding far corners */
			return 7;
		case -9: /* Moore Neighbor Count but excluding sides */
			return 8;
		case 12: /* Moore Neighbor Count */
			return 9;
		default:
			System.out.println("no neighborhood like " +
				neighbors + " known");
			return Ant2DInterface.DEFAULT_NEIGHBOR_INDEX;
		}
	}

	static String parse(String rule) {
		return neighborIndexStrings[indexToNeighbor[parseNeighborFromRule(rule)]];
		//return Integer.toHexString(turingRule);
	}

	static int parseNeighborFromRule(String rule) {
		int neighbor = 0;
		int i = 0;
		boolean negative = false;

		for (; i < rule.length(); i++) {
			char letter = rule.charAt(i);

			if (letter >= '0' && letter <= '9')
				neighbor = neighbor * BASE + (letter - '0');
			else if (letter == '-')
				negative = !negative;
			else
				break;
		}
		if (negative)
			return neighborToIndex(-neighbor);
		return neighborToIndex(neighbor);
	}

	static final int STATES = 2;
	static final int MAXCOLORS = 8;

	/* Relative ant moves */
	static final int FS   = 0;  /* Step */
	static final int TRS  = 1;  /* Turn right, then step */
	static final int THRS = 2;  /* Turn hard right, then step */
	static final int TBS  = 3;  /* Turn back, then step */
	static final int THLS = 4;  /* Turn hard left, then step */
	static final int TLS  = 5;  /* Turn left, then step */
	static final int SF   = 6;  /* Step */
	static final int STR  = 7;  /* Step then turn right */
	static final int STHR = 8;  /* Step then turn hard right */
	static final int STB  = 9;  /* Step then turn back */
	static final int STHL = 10; /* Step then turn hard left */
	static final int STL  = 11; /* Step then turn left */

/* LANGTON'S ANT (10) Chaotic after 500, Builder after 10,000 (104p) */
/* TURK'S 100 ANT Always chaotic?, tested past 150,000,000 */
/* TURK'S 101 ANT Always chaotic? */
/* TURK'S 110 ANT Builder at 150 (18p) */
/* TURK'S 1000 ANT Always chaotic? */
/* TURK'S 1100 SYMMETRIC ANT  all even run 1's and 0's are symmetric */
/* other examples 1001, 110011, 110000, 1001101 */
/* TURK'S 1101 ANT Builder after 250,000 (388p) */
/* Once saw a chess horse type builder (i.e. non-45 degree builder) */

/* BEE ONLY */
/* All alternating 10 appear symmetric, no proof (i.e. 10, 1010, etc) */
/* Even runs of 0's and 1's are also symmetric */
/* I have seen Hexagonal builders but they are more rare. */

	static final int[][] table = new int[][] {
		/* Here just so you can figure out notation */
		/* Langton's ant (-rule 10) */
		{
			2, 1,
			1, TLS, 0, 0, TRS, 0
		},
		/* First 2 numbers are the size (ncolors, nstates) */
		/* LADDER BUILDER */
		{
			4, 1,
			1, STR, 0, 2, STL, 0, 3, TRS, 0, 0, TLS, 0
		},
		/* SPIRALING PATTERN */
		{
			2, 2,
			1, TLS, 0, 0, FS, 1,
			1, TRS, 0, 1, TRS, 0
		},
		/* SQUARE (HEXAGON) BUILDER */
		{
			2, 2,
			1, TLS, 0, 0, FS, 1,
			0, TRS, 0, 1, TRS, 0
		},
	};

	FiniteStateMachine[] machine = null;

	void initMachine() {
		machine = new FiniteStateMachine[MAXCOLORS * STATES];
	}

	static int totalAngles(int neighbors) {
		if (neighbors == 7)
			return 357; // 360 not divisible by 7
		return 360;
	}

	static short fromTableDirection(int dir, int neighbors) {
		int angles = totalAngles(neighbors);
		int deflect = 2;

		if (neighbors == 8) {
			deflect = 3;
		}
		/* Crafted to work for odd number of neighbors */
		switch (dir) {
		case FS:
			return (short) 0;
		case TLS:
			return (short) (angles / neighbors);
		case THLS:
			return (short) (deflect * angles / neighbors);
		case TBS:
			return (short) ((neighbors / 2) * angles / neighbors);
		case THRS:
			return (short) (angles - deflect * angles / neighbors);
		case TRS:
			return (short) (angles - angles / neighbors);
		case SF:
			return (short) angles;
		case STL:
			return (short) (angles + angles / neighbors);
		case STHL:
			return (short) (angles + deflect * angles / neighbors);
		case STB:
			return (short) (angles + (neighbors / 2) * angles / neighbors);
		case STHR:
			return (short) (2 * angles - deflect * angles / neighbors);
		case STR:
			return (short) (2 * angles - angles / neighbors);
		default:
			System.out.println("wrong direction " + dir +
				" from table");
		}
		return (short) (2 * angles);
	}

	static boolean boring(int number) {
		int temp = number;

		while (temp >= 1) {
			if ((temp & 1) == 0) { // look for a zero bit
				return false; // well it may not be interesting
			}
			temp >>= 1;
		}
		if (DEBUG)
			System.out.println("rule modified");
		return true;
	}

	private static int log2(int number) {
		int temp = number;
		int count = 0;

		while (temp >= 1) {
			temp >>= 1;
			count++;
		}
		return count;
	}

	void parseRule(int neighbors, int number, boolean useTable,
			boolean sharpTurn) {
		int newNumber = number;

		if (newNumber < MIN_RULE)
			newNumber = MIN_RULE;
		if (useTable) {
			newNumber -= MIN_RULE;
			newNumber = newNumber % table.length;
			getTable(neighbors, newNumber, sharpTurn);
			return;
		}
		/* if someone really insists let them have it */
		//if (boring(number)) /* all ones or zeros are boring */
		//	newNumber--; /* next lowest should be ok, 2 or > */
		getTurk(neighbors, log2(newNumber), newNumber, sharpTurn);
	}

	void getTable(int neighbors, int i, boolean sharpTurn) {
		ncolors = table[i][0];
		nstates = table[i][1];
		int total = ncolors * nstates;
		byte k = 2;

		for (int j = 0; j < total; j++) {
			byte color = (byte) table[i][k++];
			int turn = table[i][k++];
			short direction;

			if (sharpTurn && neighbors > 4) {
				switch(turn) {
				case TRS:
					turn = THRS;
					break;
				case THRS:
					turn = TRS;
					break;
				case THLS:
					turn = TLS;
					break;
				case TLS:
					turn = THLS;
					break;
				case STR:
					turn = STHR;
					break;
				case STHR:
					turn = STR;
					break;
				case STHL:
					turn = STL;
					break;
				case STL:
					turn = STHL;
					break;
				default:
					break;
				}
				direction = fromTableDirection(turn, neighbors);
			} else {
				direction = fromTableDirection(turn, neighbors);
			}
			machine[j] = new FiniteStateMachine(color, direction, (byte) table[i][k++]);
		}
		//truchet = false;
	}

	void getTurk(int neighbors, int logClass, int number, boolean sharpTurn) {
		int power2, j, total;
		StringBuilder rule = new StringBuilder();

		turingRule = number;
		power2 = 1 << (logClass - 1);
		ncolors = logClass;
		nstates = 1;
		total = ncolors * nstates;
		for (j = 0; j < total; j++) {
			byte color = (byte) ((j + 1) % total);
			boolean turn = ((power2 & number) != 0);
			short direction;

			if (sharpTurn && neighbors > 4) {
				direction = ((turn) ?
					fromTableDirection((byte) THRS, neighbors) :
					fromTableDirection((byte) THLS, neighbors));
			} else {
				direction = ((turn) ?
					fromTableDirection((byte) TRS, neighbors) :
					fromTableDirection((byte) TLS, neighbors));
			}
			rule.append((turn) ? "1" : "0");
			power2 >>= 1;
			machine[j] = new FiniteStateMachine(color, direction, (byte) 0);
			if (DEBUG)
				System.out.println("color " + color +
					", direction " + direction);
		}
		//truchet = (truchet && >xs > 2 && ys > 2 &&
		//  (neighbors == 3 || neighbors == 4 || neighbors == 6));
		if (DEBUG)
			System.out.println("neighbors " + neighbors +
				", Turk's number 0x" +
				Integer.toHexString(number) + "(" +
				rule.toString() + "), colors " + ncolors);
	}

	// for debugging
	static String toString(int neighbors, /*Ant2DRules param,*/ boolean verbose) {
		StringBuilder sb = new StringBuilder("Rule");

		if (verbose)
			System.out.print("rule (" +
				neighbors + " neighborhood): ");
		//int neighborIndex = neighborToIndex(neighbors);
		if (verbose) {
			System.out.println(sb);
		}
		return sb.toString();
	}

	int getNColors() {
		return ncolors;
	}

	int getNStates() {
		return nstates;
	}
}
